/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.orchestration.core;

import java.io.StringReader;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.orchestration.ExchangeContext;
import org.ow2.petals.orchestration.Orchestrator;

public class ExchangeContextImpl implements ExchangeContext {

	private Exchange exchange_;
	private Orchestrator original_ = null;

	public ExchangeContextImpl(Exchange exchange, Orchestrator original) {
		exchange_ = exchange;
		original_ = original;
	}
	
	/* (non-Javadoc)
	 * @see org.ow2.petals.orchestration.java.ExchangeContext#isInOut()
	 */
	public boolean isInOut() {
		return (exchange_.isInOutPattern() || exchange_.isInOptionalOutPattern());
	}
	
	/* (non-Javadoc)
	 * @see org.ow2.petals.orchestration.java.ExchangeContext#reply(javax.xml.transform.Source)
	 */
	public void reply(Source content) throws MessagingException {
		if(isInOut()) exchange_.setOutMessageContent(content);
	}
	
	public void reply(String content) throws MessagingException {
		if(isInOut()) reply(new StreamSource(new StringReader(content)));
	}

	public void setFault(String content) throws MessagingException {
		Fault f = exchange_.createFault();
		f.setContent(new StreamSource(new StringReader(content)));
		exchange_.setFault(f);
	}

	public Orchestrator getOriginalOrchestrator() { return original_; }
	
}
