/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.orchestration.core;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.security.auth.Subject;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfOperation.MEPPatternConstants;
import org.ow2.easywsdl.wsdl.util.InputStreamForker;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;

public class InvocationContext implements Serializable {
		private transient Exchange exchange_;
		transient Map<String, DataHandler> attachments_ = new HashMap<String, DataHandler>();
		Map<String, Object> properties_ = new HashMap<String, Object>();
		transient Source content_;
		String endpoint_;
		String exchangeId_;
		MEPPatternConstants mep_ = MEPConstants.IN_OUT_PATTERN;
		QName operation_ = null;
		boolean isOut_;
		Subject subject_ = null;
		
		public InvocationContext() {
		}
		
		public InvocationContext(Exchange exchange) throws MessagingException {
			this();
			exchange_ = exchange;
			endpoint_ = OrchestrationUtils.getEndpointName(exchange.getEndpoint());
			exchangeId_ = exchange.getExchangeId();
			isOut_ = exchange.isOutMessage();
			content_ = extractContent(exchange);
			if(exchange.isInOnlyPattern()) mep_ = MEPConstants.IN_ONLY_PATTERN;
			else if(exchange.isRobustInOnlyPattern()) mep_ = MEPConstants.ROBUST_IN_ONLY_PATTERN;
			else if(exchange.isInOptionalOutPattern()) mep_ = MEPConstants.IN_OPTIONAL_OUT_PATTERN;
			else mep_ = MEPConstants.IN_OUT_PATTERN;
			operation_ = exchange.getOperation();
			Set<String> names = (isOut_ ? exchange.getOutMessageAttachmentNames() : exchange.getInMessageAttachmentNames());
			for(String name : names) {
				attachments_.put(name, (isOut_ ? exchange.getOutMessageAttachment(name) : exchange.getInMessageAttachment(name)));
			}
			subject_ = (isOut_ ? exchange.getOutSubject() : exchange.getInSubject());
			Set<String> pnames = exchange.getPropertyNames();
			for(final String name : pnames) {
				properties_.put(name, exchange.getProperty(name));
			}
		}
		
		public InvocationContext(InvocationContext source, ComponentContext ctx) {
			this();
			try {
				exchange_ = OrchestrationUtils.copyExchange(
						ctx.getDeliveryChannel().createExchangeFactory(), source.getOriginalExchange());
			} catch (Exception e) {
				exchange_ = source.getOriginalExchange();
				e.printStackTrace();
			}
			//exchange_ = source.getOriginalExchange();
			isOut_ = source.isOut_;
			content_ = source.getContent(false);
			mep_ = source.mep_;
			operation_ = source.operation_;
			Set<String> names = attachments_.keySet();
			for(String name : names) {
				putAttachment(name, source.getAttachment(name));
			}
		}
		
		private Source extractContent(Exchange exchange) throws MessagingException {
			boolean isOut = exchange.isOutMessage();
			Source ret = null;
			Source content = (isOut ? exchange.getOutMessageContentAsSource() : exchange.getInMessageContentAsSource());
			if(content instanceof StreamSource) {
				InputStreamForker forker = new InputStreamForker(((StreamSource)content).getInputStream());
				ret = new StreamSource(forker.getInputStreamOne());
				if(isOut) exchange.setOutMessageContent(forker.getInputStreamTwo());
				else exchange.setInMessageContent(forker.getInputStreamTwo());
			} else {
				ret = content;
			}
			return ret;
		}
		
		protected Exchange getOriginalExchange() { return exchange_; }
		
		public Source getContent() { return getContent(true); }
		
		public Source getContent(boolean consume) {
			Source ret = content_;
			if(! consume) {
				if(content_ instanceof StreamSource) {
					InputStreamForker forker = new InputStreamForker(((StreamSource)content_).getInputStream());
					ret = new StreamSource(forker.getInputStreamOne());
					content_ = new StreamSource(forker.getInputStreamTwo());
				}
			}
			return ret;
		}
		
		public void setContent(Source content) { content_ = content; }
	
		public MEPPatternConstants getMep() { return mep_; }
		public void setMep(MEPPatternConstants mep) { mep_ = mep; }

		public QName getOperation() { return operation_; }
		public void setOperation(QName operation) { operation_ = operation; }

		public void setEndpoint(String ep) { endpoint_ = ep; }
		public String getEndpoint() { return endpoint_; }

		public String getExchangeId() { return exchangeId_; }

		public DataHandler getAttachment(String name) { return attachments_.get(name); }
		public void putAttachment(String name, DataHandler attachment) { attachments_.put(name, attachment); }
		public DataHandler removeAttachment(String name) { return attachments_.remove(name); }
		public boolean hasAttachments() { return (attachments_.size() > 0); }
		public Set<String> getAttachmentNames() { return attachments_.keySet(); }
		
		public void unzipAttachment(String attname) throws Exception {
			DataHandler attachment = attachments_.get(attname);
			if(attachment == null) return;
			
			DataHandler result = OrchestrationUtils.unzip(attachment);

			if(result != null) {
				attachments_. put(attname, result);
			}
		}
		
		public void unzipAttachments() throws Exception {
			Set<String> names = getAttachmentNames();
			for(String name : names) {
				unzipAttachment(name);
			}
		}
		/**
		 * Switch attachment to payload (the previous payload gets lost, and the corresponding attachment is removed).
		 * When no attachment is eligible, nothing is done.
		 * @param attname The attachment name (null means 1st attachment).
		 */
		public void attachmentToPayload(String attname) {
				DataHandler attachment = null;
				Set<String> attnames = attachments_.keySet();
				if(! attnames.isEmpty()) {
					String name = null;
					if(attname == null) {
						name = attnames.iterator().next();
					} else {
						for(String n : attnames) {
							if(attname.equals(n)) {
								name = n;
								break;
							}
						}
					}
					
					if(name != null) {
						attachment = attachments_.remove(name);
						try {
							StreamSource content = new StreamSource(attachment.getDataSource().getInputStream());
							content_ = content;
						} catch (IOException ignore) { }
					}
				}
		}
		
		public void transform(String xslt, URIResolver r, String attname) throws Exception {
			TransformerFactory tFactory = TransformerFactory.newInstance();
		    Transformer transformer = tFactory.newTransformer(new StreamSource(new FileReader(xslt)));
		    if(r != null) transformer.setURIResolver(r);
		    Source in = content_;
		    ByteArrayOutputStream out = new ByteArrayOutputStream();
		    transformer.transform(in, new StreamResult(out));
		    //System.out.println(out);
		    
		    if(attname == null) {
		    	content_ = new StreamSource(new ByteArrayInputStream(out.toByteArray()));  	
		    } else {
		    	DataHandler attachment = new DataHandler(new MemoryDataSource(out.toByteArray(), attname, "application/octet-stream")); 
		    	attachments_.put(attname, attachment);
		    	content_ = new StreamSource(new StringReader("<attached-files><file-name>"
	                    + attname + "</file-name></attached-files>"));
		    }
		}

		public Exchange createConsumeExchange(String epr, AbstractJBIListener listener) throws Exception {
			Consumes endpoint = OrchestrationUtils.getConsumesEndpoint(epr);
			if(endpoint == null) throw new Exception("Invalid endpoint " + epr);
			Exchange out = listener.createConsumeExchange(endpoint, getMep());
		
			// Transfer content
			out.setInMessageContent(getContent(false));
			if(operation_ != null) out.setOperation(operation_);
			
			// Transfer attachments
			Set<String> names = attachments_.keySet();
			for(String name : names) {
				out.setInMessageAttachment(name, attachments_.get(name));
			}
			// Transfer headers
			if(properties_ != null) out.setInMessageProperties(properties_);
			if(subject_ != null) out.setInSubject(subject_);
			
			return out;
		}

            /**
             * Used to deserialize content + attachments
             *
             * @param s
             * @throws IOException
             */
            private void readObject(ObjectInputStream s) throws IOException {
                try {
                    s.defaultReadObject();
                    attachments_ = ICSerializer.deserializeAttachments(s);
                    content_ = ICSerializer.deserializeContent(s);
                } catch (ClassNotFoundException e) {
                    throw new IOException(e.getClass() + ":" + e.getMessage());
                }
            }

            /**
             * <p>
             * Serialization process :
             * <ul>
             * <li>Serialization of all serializable fields of the NormalizedMessage</li>
             * <li>Serialization of all attachment</li>
             * <li>Finally, serialization of the content.</li>
             * </ul>
             * </p>
             * <p>
             * <b>Note</b>: Content must always be serialized at the end of the
             * serialization process. Because in the deserialization process, a new
             * content will be created with all ending bytes of the Stream.
             * </p>
             *
             * @param s
             * @throws IOException
             */
            private void writeObject(ObjectOutputStream s) throws IOException {
                s.defaultWriteObject();
                try {
                    ICSerializer.serializeAttachments(attachments_, s);
                    ICSerializer.serializeContent(content_, s);
                } catch (final Exception e) {
                    throw (IOException) (new IOException(e.getMessage()).initCause(e));
                }
            }

}

