/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.orchestration.sample;

import org.ow2.petals.orchestration.ExchangeContext;
import org.ow2.petals.orchestration.Orchestrator;
import org.ow2.petals.orchestration.PojoScript;

public class SampleScript extends PojoScript {

	@Override
	public void onExchange(ExchangeContext ectx, Orchestrator orchestrator)
	throws Exception {
		
		/*
		 Expected message in exchange:
		 <q0:listFiles xmlns:q0="http://petals.ow2.org/components/filetransfer/version-2.2">
		 <q0:filepattern>*.xml</q0:filepattern>
		 </q0:listFiles>
		*/
		
		getLogger().fine("POJO Script: before consume");
		
		orchestrator
			.setOperation("http://petals.ow2.org/components/filetransfer/version-2.2", "getFiles")
			.inOut("{http://petals.ow2.org/components/filetransfer/version-2}GetFiles|TestFile|TestFileEndpoint")
			.clearOperation()
			.inOnly("{http://petals.ow2.org/components/filetransfer/version-2}GetFiles|TestFile|TestFileEndpoint");
		
		System.out.println("POJO Script: consume done");
		
		// Answer if needed (In[Optional]Out)
		if(ectx.isInOut()) {
			getLogger().fine("POJO Script: sending response");
			ectx.reply("<pojoScript>Ok</pojoScript>");
		}
	}
	
	@Override
	public void onAsyncExchange(ExchangeContext ectx, Orchestrator orchestrator)
	throws Exception {
		getLogger().fine("POJO Script: onAsyncExchange method called !");
	}
	
	@Override
	public void init() {
		getLogger().fine("Sample PojoScript inits.");
	}

}
